package com.yizhi.album.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.album.application.vo.domain.AlbumClassify;
import com.yizhi.album.application.vo.AlbumClassifyVo;
import com.yizhi.album.application.vo.ApiAlbumClassifyVo;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@FeignClient(name = "album", contextId = "AlbumClassifyClient")
public interface AlbumClassifyClient {

	@GetMapping("/remote/albumClassify/find/id")
	public AlbumClassify getById(@RequestParam(name = "id") Long id);

	@ApiOperation("添加分类")
	@RequestMapping(value = "/remote/albumClassify/save", method = RequestMethod.POST)
	public int saveClassify(@RequestBody AlbumClassify classify);

	/**
	 * 检查是否存在分类名称
	 * 
	 * @param name       分类名称
	 * @param parentId   分类的上级id（一级分类为0）
	 * @param siteId     站点id
	 * @param type       检查类型（1：新增，2：修改(需要传classifyId)）
	 * @param classifyId 分类id，修改检查时用到
	 * @return 不存在返回true，存放相同名返回false
	 */
	@GetMapping("/remote/albumClassify/check/name")
	public boolean checkExtisClassifyName(@RequestParam(name = "name") String name,
			@RequestParam(name = "parentId", required = false) Long parentId, @RequestParam(name="siteId") Long siteId,
			@RequestParam(name="type") Integer type, @RequestParam(name = "classifyId", required = false) Long classifyId);

	@ApiOperation("分类删除")
	@RequestMapping(value = "/remote/albumClassify/delete", method = RequestMethod.POST)
	public int deleteClassify(@RequestBody AlbumClassify classify);

	@ApiOperation("分类名称修改")
	@RequestMapping(value = "/remote/albumClassify/update", method = RequestMethod.POST)
	public int updateClassify(@RequestBody AlbumClassify classify);
	
	@ApiOperation("分类查询")
	@RequestMapping(value = "/remote/albumClassify/list", method = RequestMethod.GET)
	public Page<AlbumClassify> listClassify(@RequestParam(required = false, name = "isStartHQ") Boolean isStartHQ,
			@RequestParam(required = false,name = "name") String name, @RequestParam(name = "pageNo") Integer pageNo, @RequestParam(name = "pageSize") Integer pageSize,
			@RequestParam(name = "companyId") Long companyId, @RequestParam(name = "siteId") Long siteId,
			@RequestParam(name = "orgIds", required = false) List<Long> orgIds);
	
	@ApiOperation("分类排序")
	@RequestMapping(value = "/remote/albumClassify/sort", method = RequestMethod.POST)
	public int sortClassify(@RequestBody List<AlbumClassify> list);
	
	@ApiOperation("分类分页查询")
	@RequestMapping(value = "/remote/albumClassify/page/list", method = RequestMethod.GET)
	public Page<ApiAlbumClassifyVo> pageList(@RequestParam(name = "name", required = false) String name,
			@RequestParam(name = "pageNo") Integer pageNo, @RequestParam(name = "pageSize") Integer pageSize,
			@RequestParam(name = "companyId") Long companyId, @RequestParam(name = "siteId") Long siteId,
			@RequestParam(name = "orgIds", required = false) List<Long> orgIds);
	
	/**
	 * 验证同站点下 同级的分类不能重名
	 *
	 * @param name      分类名
	 * @param layer     分类层级
	 * @param companyId 企业名
	 * @param siteId    站点id
	 * @param id        分类id （修改的时候传入）
	 * @return Boolean
	 */
	@GetMapping("/remote/albumClassify/vail/name")
	public Boolean vailName(@RequestParam(name = "name") String name, @RequestParam(name = "layer") Integer layer,
			@RequestParam(name = "companyId") Long companyId, @RequestParam(name = "siteId") Long siteId,
			@RequestParam(name = "id", required = false) Long id);
	
	@GetMapping("/remote/albumClassify/getClassifyName")
	public String getClassifyName(@RequestParam("classifyId") Long classifyId);
	
    @GetMapping("/remote/albumClassify/classify/list")
    public List<AlbumClassifyVo> listClassifyInfo(
            @RequestParam(value="classifyId",required = false) Long classifyId);
}
