package com.yizhi.album.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.album.application.vo.*;
import com.yizhi.album.application.vo.domain.Album;
import com.yizhi.album.application.vo.domain.AlbumVisibleRange;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.vo.DroolsVo;
import io.swagger.annotations.ApiParam;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@FeignClient(name = "album", contextId = "AlbumClient")
public interface AlbumClient {

	/**
	 * 管理端专辑列表接口
	 * @param name
	 * @param alClassifyId
	 * @param status
	 * @param overed
	 * @param companyId
	 * @param siteId
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	@GetMapping("/remote/album/list")
	public Page<Album> list(@RequestParam(name = "name",required = false) String name,
			@RequestParam(name = "alClassifyId",required = false) Long alClassifyId, @RequestParam(name = "status",required = false) Integer status,
			@RequestParam(name = "overed",required = false) Integer overed, @RequestParam(name = "companyId") Long companyId,
			@RequestParam(name = "siteId") Long siteId, @RequestParam(name = "pageNo") Integer pageNo,
			@RequestParam(name = "pageSize") Integer pageSize);
	
	/**
	 * 管理端专辑详情接口
	 * @param id
	 * @return
	 */
	@GetMapping("/remote/album/get")
	public Album get(@RequestParam(name = "id") Long id);
	
	/**
	 * 管理端专辑主体新增接口
	 * @param albumVo
	 * @return
	 */
	@PostMapping("/remote/album/save")
	public AlbumVo save(@RequestBody AlbumVo albumVo);
	
	/**
	 * 管理端专辑主体修改接口
	 * @param albumVo
	 * @return
	 */
	@PostMapping("/remote/album/update")
	public Boolean update(@RequestBody AlbumVo albumVo);
	
    /**
     * 专辑下架  当专辑为上架状态 1 时 才下架
     *
     * @param albumId 参数中有专辑主键id
     * @return
     */
    @GetMapping("/remote/album/down")
    public Boolean down(@RequestParam(name = "albumId") Long albumId);
    
    /**
     * 专辑上架  
     *
     * @param albumId 参数中有专辑主键id
     * @return
     */
    @Deprecated
    @GetMapping(value = "/remote/album/up")
    public Boolean up(@RequestParam(name = "albumId") Long albumId,@RequestParam(name = "type")Integer type);

	/**
	 * 专辑上架
	 *
	 * @param albumId 参数中有专辑主键id
	 * @return
	 */
	@GetMapping(value = "/remote/album/up2")
	public Integer up2(@RequestParam(name = "albumId") Long albumId,@RequestParam(name = "type")Integer type);

	/**
     * 专辑可见范围新增
     * @param albumVisableRanges
     * @return
     */
	@PostMapping("/remote/album/authorize/insert")
	public Boolean insertAlbumVisibleRanges(@RequestBody List<AlbumVisibleRange> albumVisableRanges);
	
	/**
	 * 专辑可见范围查看
	 * @param albumId
	 * @return
	 */
	@GetMapping("/remote/album/VisibleRange")
	public List<AlbumVisibleRange> VisibleRange(@RequestParam("albumId") Long albumId);
	
	/**
	 * 专辑课程清单导出
	 * @param albumId
	 * @return
	 */
	@GetMapping("/remote/album/activity/export")
	public Map<String, Object> activityExport(@RequestParam(name = "albumId", required = true) Long albumId);
	
	/**
	 * 专辑列表查询
	 * @param name
	 * @param companyId
	 * @param siteId
	 * @return
	 */
	@GetMapping("/remote/album/searchList")
	public List<Album> searchList(@RequestParam(name = "name",required = false) String name,@RequestParam(name = "ids",required = false) List<Long> ids,
			@RequestParam(name = "companyId") Long companyId, @RequestParam(name = "siteId") Long siteId);
	
	/**
	 * 学员端专辑首页列表
	 * @param apiSearchAlbumVo
	 * @return
	 */
	@PostMapping("/remote/album/searchPage")
	public Page<ApiListAlbumVo> searchPage(@RequestBody ApiSearchAlbumVo apiSearchAlbumVo);
	
	/**
	 * 学员端我的专辑
	 * @param siteId
	 * @param accountId
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	@GetMapping("/remote/album/getMyAlbumList")
	public Page<ApiListAlbumVo> getMyAlbumList(@RequestParam("siteId") Long siteId,
			@RequestParam("accountId") Long accountId, @RequestParam(name = "pageNo") Integer pageNo,
			@RequestParam(name = "pageSize") Integer pageSize);
	
	/**
	 * 学员端专辑详情
	 * @param albumId
	 * @param accountId
	 * @param siteId
	 * @param companyId
	 * @return
	 */
	@GetMapping("/remote/album/getDeatil")
	public AlbumDetailVo getDeatil(@RequestParam("albumId") Long albumId, @RequestParam("accountId") Long accountId, @RequestParam("siteId") Long siteId, @RequestParam("companyId") Long companyId);
	
	/**
	 * 学员端专辑内容
	 * @param albumId
	 * @param accountId
	 * @param siteId
	 * @param companyId
	 * @param type
	 * @param display
	 * @return
	 */
	@GetMapping("/remote/album/getContent")
	public List<AlbumThemeVo> getContent(@RequestParam("albumId") Long albumId,
										 @RequestParam("accountId") Long accountId, @RequestParam("siteId") Long siteId,
										 @RequestParam("companyId") Long companyId, @RequestParam("type") Integer type,
										 @RequestParam("display") Integer display);
	
	/**
	 * 首页获取专辑列表
	 * @param ids
	 * @return
	 */
	@PostMapping("/remote/album/getPortalList")
	public List<ApiListAlbumVo> getPortalList(@RequestBody List<Long> ids);
	
	/**
	 * 我的订阅专辑
	 * @param vo
	 * @return
	 */
	@PostMapping("/remote/album/getMySubscribePage")
	public Page<ApiListAlbumVo> getMySubscribePage(@RequestBody ApiSearchAlbumVo vo);
	
	@PostMapping("/remote/album/delete")
	public Integer delete(@RequestBody List<Long> ids);
	
	@PostMapping("/remote/album/getMyAlbumCount")
	public Integer getMyAlbumCount(@RequestBody RequestContext context);
	
	@PostMapping("/remote/album/getSubcribeNum")
	public Integer getSubcribeNum(@RequestBody List<Long> ids);

	@GetMapping("/remote/album/getPageByDrools")
	Page<DroolsVo> getPageByDrools(@RequestParam("field") String field,
								   @RequestParam(value = "value", required = false) String value,
								   @RequestParam("pageNo") Integer pageNo,
								   @RequestParam("pageSize") Integer pageSize);

	/**
	 * 日历任务获取专辑列表
	 * @return
	 */
	@PostMapping("/remote/album/getPageToCalendar")
	public Page<AlbumCalendarVo> getPageToCalendar(@ApiParam("paramVo") @RequestBody CalendarTaskParamVo paramVo);
	
	@PostMapping("/remote/album/listNoIds")
	public Page<Album> listNoIds(@RequestBody ApiSearchAlbumVo vo);
}

