package com.yizhi.album.application.controller.remote;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.album.application.domain.*;
import com.yizhi.album.application.mapper.AlStudentActivityRecordMapper;
import com.yizhi.album.application.mapper.AlbumMapper;
import com.yizhi.album.application.service.IAlThemeActivityService;
import com.yizhi.album.application.service.IAlbumClassifyService;
import com.yizhi.album.application.service.IAlbumService;
import com.yizhi.album.application.service.IAlbumVisbleRangeService;
import com.yizhi.album.application.task.ActivityListExport;
import com.yizhi.album.application.vo.*;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.comment.application.feign.SubscribeClient;
import com.yizhi.comment.application.vo.domain.SubscribeVo;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.course.application.feign.CoursePcClient;
import com.yizhi.site.application.feign.PortalManagePCFeignClients;
import com.yizhi.util.application.constant.TpActivityType;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 专辑表 前端控制器
 * </p>
 *
 * @author hc123
 * @since 2019-12-30
 */
@RestController("albumController1")
@RequestMapping("/remote/album")
public class AlbumController {

	private Logger logger = LoggerFactory.getLogger(AlbumController.class);

	@Autowired
	private IAlbumService albumService;

	@Autowired
	private IAlbumVisbleRangeService albumVisbleRangeService;

	@Autowired
	private IAlThemeActivityService alThemeActivityService;

	@Autowired
	private ActivityListExport activityListExport;
	@Autowired
	IdGenerator idGenerator;

	@Autowired
	private IAlbumClassifyService classifyService;

	@Autowired
	private SubscribeClient subscribeClient;

    @Autowired
    private RedisCache redisCache;
    
    @Autowired
    private CoursePcClient coursePcClient;
    
    @Autowired
    private AlbumMapper albumMapper;
    
    @Autowired
    private AlStudentActivityRecordMapper alStudentActivityRecordMapper;
    
    @Autowired
    private PortalManagePCFeignClients portalManagePCFeignClients;
	@GetMapping("/list")
	public Page<Album> list(@RequestParam(name = "name",required = false) String name,
                            @RequestParam(name = "alClassifyId",required = false) Long alClassifyId, @RequestParam(name = "status",required = false) Integer status,
                            @RequestParam(name = "overed",required = false) Integer overed, @RequestParam(name = "companyId") Long companyId,
                            @RequestParam(name = "siteId") Long siteId, @RequestParam(name = "pageNo") Integer pageNo,
                            @RequestParam(name = "pageSize") Integer pageSize) {
		Page<Album> page = albumService.searchPage(name, alClassifyId, status, overed, companyId, siteId, pageNo,
				pageSize);
		return page;
	}

	@GetMapping("/get")
	public Album get(@RequestParam(name = "id") Long id) {
		Album album = albumService.selectById(id);
		return album;
	}

	@PostMapping("/save")
	public AlbumVo save(@RequestBody AlbumVo albumVo) {

		return albumService.save(albumVo);
	}

	@PostMapping("/update")
	public Boolean update(@RequestBody AlbumVo albumVo) {

		return albumService.update(albumVo);
	}

	@PostMapping("/delete")
	public Integer delete(@RequestBody List<Long> ids) {
		RequestContext context = ContextHolder.get();
		logger.info("选中需要删除的专辑id={}", ids);
		List<Album> list = new ArrayList<Album>();
		for (Long id : ids) {
			Album album = albumService.selectById(id);
			if (album.getShelves() == 1) {
				return 1;
			} else {
				album.setDelFlg(1);
				album.setUpdateById(context.getAccountId());
				album.setUpdateByName(context.getAccountName());
				album.setUpdateTime(new Date());
				list.add(album);
			}
		}
		albumService.updateBatchById(list);
		return 0;
	}

	/**
	 * 专辑下架 当专辑为上架状态 1 时 才下架
	 *
	 * @param album 参数中有专辑主键id
	 * @return
	 */
	@GetMapping("/down")
	public Boolean down(@RequestParam(name = "albumId") Long albumId) {
		RequestContext context = ContextHolder.get();
		Album album2 = albumService.selectById(albumId);
		if (StringUtils.isEmpty(album2)) {
			return Boolean.FALSE;
		}
		if (album2.getShelves() == 1) {
			album2.setShelves(0);
			albumService.updateById(album2);
			return Boolean.TRUE;
		}
		return Boolean.FALSE;
	}

	@GetMapping(value = "/up2")
	public Integer up(@RequestParam(name = "albumId") Long albumId,@RequestParam(name = "type")Integer type)  {
		// 检查是否有主题活动（课程）
		RequestContext context = ContextHolder.get();
		Long accountId = context.getAccountId();
		String name = context.getAccountName();
		Date date = new Date();
		if (type.equals(1)) {
			AlThemeActivity activity = new AlThemeActivity();
			activity.setAlbumId(albumId);
			activity.setDeleted(0);
			EntityWrapper<AlThemeActivity> wrapper = new EntityWrapper<AlThemeActivity>(activity);
			List<AlThemeActivity> list = alThemeActivityService.selectList(wrapper);
			for (AlThemeActivity activity2 : list) {
				if (activity2.getUnlock().equals(1)) {
					activity2.setUnlock(0);
					activity2.setUpdateById(accountId);
					activity2.setUpdateByName(name);
					activity2.setUpdateTime(date);
					activity2.setUnlockTime(date);
					alThemeActivityService.updateById(activity2);
					Album album = albumService.selectById(albumId);
					album.setUpdateById(accountId);
					album.setUpdateByName(name);
					album.setUpdateTime(date);
					albumService.updateById(album);
				}
			}
			if (alThemeActivityService.selectCount(new EntityWrapper<>(activity)) < 1) {
				return 3;
			}
			Album album = albumService.selectById(albumId);
			if (album.getShelves() == 0 || album.getShelves() == 2) {
				album.setShelves(1);
				albumService.updateById(album);
				return 1;
			}
		}else {
			Album album = albumService.selectById(albumId);
			album.setShelves(2);
			albumService.updateById(album);
			return 1;
		}

		return 2;
	}

	@PostMapping("/authorize/insert")
	public Boolean insertAlbumVisibleRanges(@RequestBody List<AlbumVisibleRange> albumVisableRanges) {
		if (!CollectionUtils.isEmpty(albumVisableRanges)) {
			RequestContext context = ContextHolder.get();
			Long albumId = albumVisableRanges.get(0).getAlbumId();
			AlbumVisibleRange albumVisableRange = new AlbumVisibleRange();
			albumVisableRange.setState(1);
			albumVisableRange.setAlbumId(albumId);
			EntityWrapper<AlbumVisibleRange> entityWrapper = new EntityWrapper<>(albumVisableRange);
			albumVisbleRangeService.delete(entityWrapper);
			for (AlbumVisibleRange albumVisableRange1 : albumVisableRanges) {
				albumVisableRange1.setId(idGenerator.generate());
				albumVisableRange1.setSiteId(context.getSiteId());
				albumVisableRange1.setCompanyId(context.getCompanyId());
			}
			return albumVisbleRangeService.insertBatch(albumVisableRanges);
		} else {
			logger.error("列表为空 ");
			return Boolean.FALSE;
		}
	}

	@GetMapping("/VisibleRange")
	public List<AlbumVisibleRange> VisibleRange(@RequestParam("albumId") Long albumId) {
		AlbumVisibleRange range = new AlbumVisibleRange();
		range.setAlbumId(albumId);
		EntityWrapper<AlbumVisibleRange> wrapper = new EntityWrapper<AlbumVisibleRange>(range);
		List<AlbumVisibleRange> list = albumVisbleRangeService.selectList(wrapper);
		return list;
	}

	@GetMapping("/activity/export")
	public Map<String, Object> activityExport(@RequestParam(name = "albumId", required = true) Long albumId) {
		RequestContext requestContext = ContextHolder.get();
		Date submitTime = new Date();
		Long taskId = submitTime.getTime();
		String serialNo = "USER-EXPORT-" + taskId;
		String taskName = "专辑课程清单-" + taskId;
		String result = "任务编号：" + serialNo + "。任务名称：" + taskName;
		Album album = albumService.selectById(albumId);
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("accountId", requestContext.getAccountId());
		map.put("siteId", requestContext.getSiteId());
		map.put("companyId", requestContext.getCompanyId());
		map.put("serialNo", serialNo);
		map.put("taskId", taskId);
		map.put("submitTime", submitTime);
		map.put("taskName", taskName);
		map.put("albumId", album.getId());
		map.put("name", album.getName());
		map.put("result", result);
		activityListExport.execute(map, true);
		return map;
	}

	@GetMapping("/searchList")
	public List<Album> searchList(@RequestParam(name = "name",required = false) String name,@RequestParam(name = "ids",required = false) List<Long> ids,
			@RequestParam(name = "companyId") Long companyId, @RequestParam(name = "siteId") Long siteId) {
		return albumService.searchlList(name,ids, companyId, siteId);
	}

	@PostMapping("/searchPage")
	public Page<ApiListAlbumVo> searchPage(@RequestBody ApiSearchAlbumVo apiSearchAlbumVo) {
		logger.info("微信端专辑分类进入，查询专辑列表参数={}", JSON.toJSONString(apiSearchAlbumVo));
		List<Long> classifyIds = null;
		Long classifyId = apiSearchAlbumVo.getClassifyId();
		if (Objects.nonNull(classifyId) && classifyId.longValue() > 0) {
			classifyIds = new ArrayList<>();
			// 查询第一级的子分类
			AlbumClassify classify = classifyService.selectById(classifyId);
			Integer layer = classify.getLayer();
			classifyIds.add(classifyId);
			if (Objects.nonNull(layer) && layer.intValue() < 3) {
				AlbumClassify classify1 = new AlbumClassify();
				classify1.setParentId(classifyId);
				classify1.setStatus(1);
				List<AlbumClassify> firstIds = classifyService.selectList(QueryUtil.condition(classify1));
				Set<Long> lastIds = null;
				if (!CollectionUtils.isEmpty(firstIds)) {
					lastIds = firstIds.stream().map(objfirst -> objfirst.getId()).collect(Collectors.toSet());
					classifyIds.addAll(lastIds);
				}
				// 查询第二级的子分类
				if (Objects.nonNull(lastIds) && layer.intValue() < 2) {
					EntityWrapper<AlbumClassify> ew = QueryUtil.condition(new AlbumClassify());
					ew.in("parent_id", lastIds);
					ew.eq("status", 1);
					List<AlbumClassify> dataList = classifyService.selectList(ew);
					if (!CollectionUtils.isEmpty(dataList)) {
						classifyIds
								.addAll(dataList.stream().map(objLast -> objLast.getId()).collect(Collectors.toSet()));
					}
				}
			}
		}
		Page<ApiListAlbumVo> apiListAlbumVos = albumService.searchPageSubClassify(apiSearchAlbumVo.getName(),
				classifyIds, apiSearchAlbumVo.getIds(), apiSearchAlbumVo.getCompanyId(), apiSearchAlbumVo.getSiteId(),
				apiSearchAlbumVo.getPageNo(), apiSearchAlbumVo.getPageSize());
		return apiListAlbumVos;
	}


	@GetMapping("/getMyAlbumList")
	public Page<ApiListAlbumVo> getMyAlbumList(@RequestParam("siteId") Long siteId,
			@RequestParam("accountId") Long accountId, @RequestParam(name = "pageNo") Integer pageNo,
			@RequestParam(name = "pageSize") Integer pageSize) {
		Page<ApiListAlbumVo> page = new Page<ApiListAlbumVo>();
		page.setCurrent(pageNo);
		page.setSize(pageSize);
		List<Long> ids = albumService.getMyStudyList(siteId, accountId,new Page<Long>(pageNo, pageSize));
		List<ApiListAlbumVo> list = new ArrayList<ApiListAlbumVo>();
		for (Long id : ids) {
			ApiListAlbumVo vo = new ApiListAlbumVo();
			Album album = albumService.selectById(id);
			if (album.getShelves().equals(1)) {
				vo.setId(id);
				vo.setName(album.getName());
				vo.setImage(album.getImage());
				vo.setOvered(album.getOvered());
				vo.setUnLockTime(album.getUpdateTime());
				if (album.getUpdateTime()!=null) {
					long day = ((System.currentTimeMillis()-album.getUpdateTime().getTime())/(3600*24*1000));
					vo.setIsNew(day>5?0:1);
				}else {
					vo.setIsNew(0);
				}
				
				if (album.getOvered().equals(1)) {
					// 设置完结状态下专辑内课程数
					vo.setCourseNum(0);
					Long albumTotalCourseCount = albumMapper.getAlbumTotalCourseCount(id);
					if (null != albumTotalCourseCount){
						vo.setCourseNum(albumTotalCourseCount.intValue());
					}

					// 计算完成百分比
					List<Long> activities = albumMapper.getIdsByAlbumId(id);
					Integer finished = albumService.finished(activities,id, accountId, siteId);
					Integer unLockNum = albumService.unLockNum(id);
					if (unLockNum!=0) {
						BigDecimal res = new BigDecimal(0);
						BigDecimal dividendN = new BigDecimal(finished.toString());
						BigDecimal divisorN = new BigDecimal(unLockNum.toString());
						BigDecimal a = dividendN.multiply(new BigDecimal("100"));
						res = a.divide(divisorN, 2, BigDecimal.ROUND_HALF_UP);
						vo.setPercent(res.intValue());
					}else {
						vo.setPercent(0);
					}
				}

				list.add(vo);
			}
		}
		page.setRecords(list);
		page.setTotal(albumMapper.getMyAlbumNum(siteId, accountId));
		return page;
	}

	@GetMapping("/getDeatil")
	public AlbumDetailVo getDeatil(@RequestParam("albumId") Long albumId, @RequestParam("accountId") Long accountId,
								   @RequestParam("siteId") Long siteId, @RequestParam("companyId") Long companyId) {
		Album album = albumService.selectById(albumId);
		AlbumDetailVo vo = null;
		if (album != null) {
			if (album.getShelves()==1) {
				vo = new AlbumDetailVo();
				vo.setId(album.getId());
				vo.setName(album.getName());
				vo.setImage(album.getImage());
				vo.setClassifyId(album.getClassifyId());
				vo.setUnLockTime(album.getUpdateTime());
				vo.setOvered(album.getOvered());
				vo.setCommentEnable(album.getCommentEnable());
				vo.setDocumentEnable(album.getDocumentEnable());
				vo.setDisplay(album.getDisplay());
				vo.setDescription(album.getDescription());
				vo.setTags(album.getTags());
				if (album.getOvered().equals(1)) {
					vo.setCourseNum(0);
					Long albumTotalCourseCount = albumMapper.getAlbumTotalCourseCount(albumId);
					if (null != albumTotalCourseCount){
						vo.setCourseNum(albumTotalCourseCount.intValue());
					}
					// 计算完成百分比
					List<Long> ids = albumMapper.getIdsByAlbumId(albumId);
					Integer finished = albumService.finished(ids,albumId, accountId, siteId);
					Integer unLockNum = albumService.unLockNum(albumId);
					if (unLockNum!=0) {
						BigDecimal res = new BigDecimal(0);
						BigDecimal dividendN = new BigDecimal(finished.toString());
						BigDecimal divisorN = new BigDecimal(unLockNum.toString());
						BigDecimal a = dividendN.multiply(new BigDecimal("100"));
						res = a.divide(divisorN, 2, BigDecimal.ROUND_HALF_UP);
						vo.setPercent(res.intValue());
					}else {
						vo.setPercent(0);
					}
				}

				SubscribeVo subscribe = subscribeClient.get(albumId, TpActivityType.TYPE_ALBUM, accountId, siteId, companyId);
				vo.setSubscribe(subscribe.getStatus());
				vo.setStudyNum(albumService.getStudyNum(albumId));
				vo.setSubscribeNum(subscribeClient.getSubscribeNum(albumId, TpActivityType.TYPE_ALBUM));
			}
		}
		return vo;
	}

	@GetMapping("/getContent")
	public List<AlbumThemeVo> getContent(@RequestParam("albumId") Long albumId,
										 @RequestParam("accountId") Long accountId, @RequestParam("siteId") Long siteId,
										 @RequestParam("companyId") Long companyId, @RequestParam("type") Integer type,
										 @RequestParam("display") Integer display) {
		List<AlbumThemeVo> list = new ArrayList<AlbumThemeVo>();
		List<AlbumThemeVo> themes = albumService.getThemes(albumId, type);
		for(AlbumThemeVo themeVo:themes) {
			AlThemeActivity activity = new AlThemeActivity();
			activity.setAlbumId(albumId);
			activity.setThemeId(themeVo.getId());
			activity.setDeleted(0);
			if (!display.equals(0)) {
				activity.setUnlock(0);
			}
			EntityWrapper<AlThemeActivity> wrapper = new EntityWrapper<AlThemeActivity>(activity);
			wrapper.orderBy("sort", type.equals(1)?true:false);
			List<AlThemeActivity> activitys = alThemeActivityService.selectList(wrapper);
			List<AlbumActivityVo> list2 = new ArrayList<AlbumActivityVo>();
			for(AlThemeActivity one:activitys) {
				AlStudentActivityRecord record = new AlStudentActivityRecord();
				record.setAccountId(accountId);
				record.setSiteId(siteId);
				record.setRelationId(one.getRelationId());
				Integer count = alStudentActivityRecordMapper.selectCount(new EntityWrapper<AlStudentActivityRecord>(record));
				AlbumActivityVo vo = new AlbumActivityVo();
				vo.setAlbumId(one.getAlbumId());
				vo.setUnlock(one.getUnlock());
				vo.setId(one.getId());
				vo.setRelationId(one.getRelationId());
				vo.setName(one.getName());
				vo.setSort(one.getSort());
				vo.setUpdateTime(one.getUpdateTime());
				long firstDayBetween = (System.currentTimeMillis() - vo.getUpdateTime().getTime()) / (60 * 60 * 24 * 1000);
				if (firstDayBetween<=5) {
					vo.setIsNew(1);
					themeVo.setIsNew(1);
				}else {
					vo.setIsNew(0);
				}
				Integer status = alStudentActivityRecordMapper.getStatus(albumId, accountId, siteId, one.getRelationId());
				if (status!=null) {
					vo.setStatus(status.equals(0)?"进行中":"已完成");
				}else {
					vo.setStatus("未开始");
				}
				List<Long> courseId = new ArrayList<Long>();
				courseId.add(one.getRelationId());
        		Map<Long, Long> durations = coursePcClient.getMaterialTimeByCourseIds(courseId);
        		Long duration = (long) 0;
                if (null != durations && durations.size() > 0) {
                    for (Long h : durations.keySet()) {
                        logger.info("课程时长：" + durations.get(h));
                        duration = duration + (durations.get(h) != null ? durations.get(h) : 0l);
                    }
                }
                vo.setDuration(duration);
                list2.add(vo);
			}
			themeVo.setList(list2);
			list.add(themeVo);
		}
		return list;
	}
	
	@PostMapping("/getPortalList")
	public List<ApiListAlbumVo> getPortalList(@RequestBody List<Long> ids){
		List<ApiListAlbumVo> data = new ArrayList<ApiListAlbumVo>();
		if (!CollectionUtils.isEmpty(ids)) {
			for(Long id:ids) {
				ApiListAlbumVo vo = new ApiListAlbumVo();
				Album album = albumService.selectById(id);
				vo.setId(album.getId());
				vo.setName(album.getName());
				vo.setImage(album.getImage());
				vo.setOvered(album.getOvered());
				vo.setUnLockTime(album.getUpdateTime());
				Integer overed = album.getOvered();
				vo.setCourseNum(0);
				if (null != overed && overed.equals(1)){
					//获取已完结状态下的所有课程数量
					Long totalCourseCount = albumMapper.getAlbumTotalCourseCount(vo.getId());
					if (null != totalCourseCount){
						vo.setCourseNum(totalCourseCount.intValue());
					}
				}
				List<Long> courseIds = albumMapper.getCourseIds(vo.getId());
        		Map<Long, Long> durations = coursePcClient.getMaterialTimeByCourseIds(courseIds);
        		Long duration = (long) 0;
                if (null != durations && durations.size() > 0) {
                    for (Long h : durations.keySet()) {
                        logger.info("课程时长：" + durations.get(h));
                        duration = duration + (durations.get(h) != null ? durations.get(h) : 0l);
                    }
                }
                vo.setDuration(duration);
                data.add(vo);
			}
            List<Long> albumIds = data.stream().map(ApiListAlbumVo::getId).collect(Collectors.toList());
            List<ApiListAlbumVo> vos = alStudentActivityRecordMapper.getStudentNumByIds(albumIds);
            logger.info("专辑学习人数={}", JSON.toJSONString(vos));
            if (!CollectionUtils.isEmpty(vos)) {
                Map<Long, Integer> dataMap = vos.stream().collect(Collectors.
                        toMap(key -> key.getId(), val -> val.getStudyNum()));
                for (ApiListAlbumVo itemAlbum : data) {
                	itemAlbum.setStudyNum(dataMap.get(itemAlbum.getId()));
                }
            }
		}
		return data;
	}
	
	@PostMapping("/getMySubscribePage")
	public Page<ApiListAlbumVo> getMySubscribePage(@RequestBody ApiSearchAlbumVo vo){
		List<Long> ids = vo.getIds();
		Integer pageNo = vo.getPageNo();
		Integer pageSize = vo.getPageSize();
        Page<ApiListAlbumVo> page = albumService.getMySubscribePage(ids, pageNo, pageSize);
        return page;
	}
	
	@PostMapping("/getMyAlbumCount")
	public Integer getMyAlbumCount(@RequestBody RequestContext context) {
		return albumMapper.getMyAlbumNum(context.getSiteId(), context.getAccountId());
	}
	
	@PostMapping("/getSubcribeNum")
	public Integer getSubcribeNum(@RequestBody List<Long> ids) {
		return albumMapper.getSubscribeNum(ids);
	}

	@GetMapping("/getPageByDrools")
	Page<DroolsVo> getPageByDrools(@RequestParam("field") String field,
								   @RequestParam(value = "value", required = false) String value,
								   @RequestParam("pageNo") Integer pageNo,
								   @RequestParam("pageSize") Integer pageSize) {
		Page<DroolsVo> page = new Page<>(pageNo, pageSize);
		return albumService.getPageByDrools(field, value, page);
	}

	@PostMapping("/getPageToCalendar")
	public Page<AlbumCalendarVo> getPageToCalendar(@ApiParam("paramVo") @RequestBody CalendarTaskParamVo paramVo) {
		Page<Album> page = new Page(paramVo.getPageNo(), paramVo.getPageSize());

		return albumService.getPageToCalendar(paramVo.getDate(),page);
	}
	
	@PostMapping("/listNoIds")
	public Page<Album> listNoIds(@RequestBody ApiSearchAlbumVo vo) {
		Page<Album> page = new Page(vo.getPageNo(),vo.getPageSize());
		List<Album> data = albumMapper.selectAlbumPage(vo.getName(), vo.getIds(), vo.getCompanyId(), vo.getSiteId(), page);
		page.setRecords(data);
		return page;
	}
}
