package com.yizhi.album.application.event;


import com.yizhi.album.application.domain.AlStudentActivityRecord;
import com.yizhi.album.application.mapper.AlStudentActivityRecordMapper;
import com.yizhi.album.application.mapper.AlThemeActivityMapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.cache.CacheNamespace;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.util.application.event.TrainingProjectEvent;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Transactional
public class AlEventHandler {

	private static final Logger LOGGER = LoggerFactory.getLogger(AlEventHandler.class);

	@Autowired
	private IdGenerator idGenerator;

	@Autowired
	private RedisCache redisCache;

	@Autowired
	private AlStudentActivityRecordMapper alStudentActivityRecordMapper;
	
	@Autowired
	private AlThemeActivityMapper alThemeActivityMapper;
	
	public void handle(EventWrapper<TrainingProjectEvent> ew) {
		TrainingProjectEvent event = ew.getData();
		LOGGER.info("接收到消息：{}", event);
		List<Long> list = alThemeActivityMapper.getIdsByActivityId(event.getId(), event.getSiteId());
		if (CollectionUtils.isNotEmpty(list)) {
			for(Long id :list) {
				AlStudentActivityRecord aar = new AlStudentActivityRecord();
				aar.setId(idGenerator.generate());
				aar.setAccountId(event.getAccountId());
				aar.setAlbumId(id);
				aar.setFinishDate(event.getNow());
				aar.setFinished(event.getCourseFinished() == null ? 0 : event.getCourseFinished() ? 1 : 0);
				aar.setSeconds(event.getCourseSeconds());
				aar.setRelationId(event.getId());
				aar.setSiteId(event.getSiteId());
				LOGGER.info("入库主题活动完成记录：{}", aar);
				Integer result = alStudentActivityRecordMapper.insert(aar);
				// 缓存完成记录
				if (result.equals(1)) {
					if (aar.getFinished().equals(1)) {
						cacheRecord(CacheNamespace.AL_ACTIVITY_FINISHED, aar.getAccountId(), aar.getRelationId(), id,1);
						cacheDeleteUnfinished(aar.getAccountId(), aar.getRelationId(),id);
					} else {
						cacheRecord(CacheNamespace.AL_ACTIVITY_UNFINISHED, aar.getAccountId(), aar.getRelationId(),id, 0);
					}
				}
			}
		}
	}

	/**
	 * 缓存学习记录
	 *
	 * @param keyPrefix
	 * @param accountId
	 * @param id        planId or relationId
	 */
	private void cacheRecord(String keyPrefix, long accountId, long id,long albumId, Integer finished) {
		redisCache.hsetIfAbsent(keyPrefix.concat(String.valueOf(accountId+":"+albumId)), String.valueOf(id),
				String.valueOf(finished));
	}

	/**
	 * 删除已完成中曾经的未完成记录
	 *
	 * @param accountId
	 * @param relationId
	 */
	private void cacheDeleteUnfinished(long accountId, long relationId,long albumId) {
		redisCache.hdel(CacheNamespace.AL_ACTIVITY_UNFINISHED.concat(String.valueOf(accountId+":"+albumId)),
				new String[] { String.valueOf(relationId) });
	}

}
