package com.yizhi.album.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.yizhi.album.application.domain.AlbumClassify;
import com.yizhi.album.application.vo.AlbumClassifyVo;
import com.yizhi.album.application.vo.ApiAlbumClassifyVo;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.session.RowBounds;

import java.util.List;

/**
 * <p>
 * 专辑分类 Mapper 接口
 * </p>
 *
 * @author hc123
 * @since 2019-12-30
 */
public interface AlbumClassifyMapper extends BaseMapper<AlbumClassify> {

    /**
     * 查找这个父级分类下的最大排序数
     * @param parentId
     * @return
     */
    int selectSort(@Param("ParentId") Long parentId);
    
    /**
     *查找这个分类下的层数
     * @param parentId
     * @return
     */
    @Select("select ifnull(layer,0)\n" +
            "from cloud_album.album_classify\n" +
            "where parent_id=#{ParentId}\n" +
            "limit 1")
    int selectLayer(@Param("ParentId") Long parentId);

    /**
     * 根据id查找层级
     * @param Id
     * @return
     */
    int selectParentLayer(@Param("Id") Long Id);

    /**
     * 根据id查找root_id
     * @param Id
     * @return
     */
    Long selectRootid(@Param("Id") Long Id);

    /**
     * 根据id查找其有没有子级分类
     * @param Id
     * @return
     */
    Long selectChild(@Param("Id") Long Id);

    /**
     * 根据分类id查找下面所挂的专辑
     * @param Id
     * @return
     */
    Long selectAlbum(@Param("Id") Long Id, @Param("siteId") Long siteId);

    List<ApiAlbumClassifyVo> selectListByCompanyId(@Param("companyId") Long companyId, @Param("name") String name,
                                                   @Param("orgIds") List<Long> orgIds, @Param("siteId") Long siteId,
                                                   RowBounds rowBounds);

    Integer selectCountByCompanyId(@Param("companyId") Long companyId,@Param("name") String name,@Param("orgIds") List<Long> orgIds,@Param("siteId") Long siteId);

    List<AlbumClassify> selectClassifyByRootId(@Param("ids") List<Long> ids);


    List<AlbumClassify> selectClassifyByLayer(@Param("companyId") Long companyId,@Param("siteId") Long siteId,@Param("layer") Integer layer);


    @Select("<script>" +
            "SELECT parent_id AS id ,COUNT(0) AS total FROM album_classify " +
            "WHERE parent_id in " +
            "<foreach collection=\"classifyIds\" open=\"(\" close=\")\" separator=\",\" item=\"cid\" index=\"index\">"+
            "#{cid}"+
            "</foreach>"+
            " GROUP BY parent_id" +
            "</script>")
    List<AlbumClassifyVo> findExistChildNodes(@Param("classifyIds") List<Long> classifyIds);
}
