package com.yizhi.album.application.task;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.album.application.domain.AlTheme;
import com.yizhi.album.application.domain.AlThemeActivity;
import com.yizhi.album.application.service.IAlThemeActivityService;
import com.yizhi.album.application.service.IAlThemeService;
import com.yizhi.album.application.vo.AlThemeVo;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class ActivityListExport extends AbstractDefaultTask<String, Map<String, Object>> {

	private static final Logger logger = LoggerFactory.getLogger(ActivityListExport.class);

	@Autowired
	private IAlThemeActivityService alThemeActivityService;

	@Autowired
	private IAlThemeService alThemeService;

	@Override
	protected String execute(Map<String, Object> arg0) {
		Long accountId = (Long) arg0.get("accountId");
		Long siteId = (Long) arg0.get("siteId");
		Long companyId = (Long) arg0.get("companyId");
		Long taskId = (Long) arg0.get("taskId");
		Date submitTime = (Date) arg0.get("submitTime");
		String serialNo = (String) arg0.get("serialNo");
		String taskName = (String) arg0.get("taskName");
		String name = (String) arg0.get("name");
		Long albumId = (Long) arg0.get("albumId");
		/**
		 * 走异步任务
		 */
		TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, submitTime, siteId, companyId);
		working(taskContext);
		String upLoadUrl = null;
		String requestPath = FileConstant.SAVE_PATH;
		File fileDir = new File(requestPath);
		if (!fileDir.exists()) {
			fileDir.mkdir();
		}
		// 课程清单开始组装
		AlTheme theme = new AlTheme();
		theme.setAlbumId(albumId);
		theme.setDeleted(0);
		EntityWrapper<AlTheme> wrapper = new EntityWrapper<AlTheme>(theme);
		wrapper.orderBy("sort", true);
		List<AlTheme> themes = alThemeService.selectList(wrapper);
		List<AlThemeActivity> list = new ArrayList<AlThemeActivity>();
		for (AlTheme alTheme : themes) {
			AlThemeVo themeVo = new AlThemeVo();
			themeVo.setAlbumId(albumId);
			themeVo.setId(alTheme.getId());
			themeVo.setName(alTheme.getName());
			AlThemeActivity activity = new AlThemeActivity();
			activity.setThemeId(alTheme.getId());
			activity.setDeleted(0);
			EntityWrapper<AlThemeActivity> entityWrapper = new EntityWrapper<AlThemeActivity>(activity);
			entityWrapper.orderBy("sort", true);
			List<AlThemeActivity> activities = alThemeActivityService.selectList(entityWrapper);
			activities.forEach(e->e.setThemeName(alTheme.getName()));
			list.addAll(activities);
		}
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
		SimpleDateFormat ff = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");		
		SimpleDateFormat sf = new SimpleDateFormat("yyyyMMddHHmmss"); 	 	
		try {
			//excel生成过程: excel-->sheet-->row-->cell
			// 第一步，创建一个Excel文件
			XSSFWorkbook wb = new XSSFWorkbook();
			// 第二步，在webbook中添加一个sheet,对应Excel文件中的sheet
			XSSFSheet sheet = wb.createSheet(name+"-课程清单"+sf.format(submitTime));
			sheet.setDefaultColumnWidth(56);
			XSSFFont font = wb.createFont();
			font.setFontName("宋体");
			font.setFontHeightInPoints((short) 16);
			font.setBold(false);
			XSSFCellStyle style = wb.createCellStyle();
			style.setFont(font);
			style.setAlignment(HorizontalAlignment.LEFT); // 创建一个左对齐格式
			style.setVerticalAlignment(VerticalAlignment.CENTER);
			// 第三步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short
			XSSFRow row = sheet.createRow((int) 0);
			row.setHeightInPoints((float) 21);
			// 合并单元格  
			CellRangeAddress cra=new CellRangeAddress(0,0,0,3); // 起始行, 终止行, 起始列, 终止列  
			sheet.addMergedRegion(cra);
			XSSFCell cell = row.createCell((short) 0);
			cell.setCellValue(name+"-课程清单"+sf.format(submitTime));
			cell.setCellStyle(style);
			XSSFRow row1 = sheet.createRow((int) 1);
			row1.setHeightInPoints((float) 18);
			// 合并单元格  
			CellRangeAddress cra1=new CellRangeAddress(1,1,0,3); // 起始行, 终止行, 起始列, 终止列  
			sheet.addMergedRegion(cra1);
			XSSFCell cell1 = row1.createCell((short) 0);
			cell1.setCellValue("报表生成时间："+ff.format(submitTime));
			XSSFCellStyle style1 = wb.createCellStyle();
			XSSFFont font1 = wb.createFont();
			font1.setFontName("宋体");
			font1.setFontHeightInPoints((short) 14);
			font1.setBold(false);
			style1.setFont(font1);
			style1.setAlignment(HorizontalAlignment.LEFT);
			style1.setVerticalAlignment(VerticalAlignment.CENTER);
			cell1.setCellStyle(style1);
			XSSFRow row2 = sheet.createRow((int) 2);
			XSSFCellStyle style2 = wb.createCellStyle();
			XSSFFont font2 = wb.createFont();
			font2.setFontName("宋体");
			font2.setFontHeightInPoints((short) 12);
			font2.setBold(true);
			style2.setFont(font2);
			style2.setAlignment(HorizontalAlignment.CENTER);
			style2.setVerticalAlignment(VerticalAlignment.CENTER);
			row2.setHeightInPoints((float) 15.6);
			row2.createCell((short) 0).setCellValue("课程名称");
			row2.getCell(0).setCellStyle(style2);
			row2.createCell((short) 1).setCellValue("解锁状态"); 
			row2.getCell(1).setCellStyle(style2);
			row2.createCell((short) 2).setCellValue("主题名称"); 
			row2.getCell(2).setCellStyle(style2);
			row2.createCell((short) 3).setCellValue("课程编码"); 
			row2.getCell(3).setCellStyle(style2);
			AlThemeActivity vo = new AlThemeActivity();
				for(int i=0;i<list.size();i++) {
					XSSFCellStyle style3 = wb.createCellStyle();
					XSSFFont font3 = wb.createFont();
					font3.setBold(false);
					font3.setFontHeightInPoints((short) 10);
					style3.setFont(font3);
					style3.setAlignment(HorizontalAlignment.CENTER); // 创建一个居中格式
					style3.setVerticalAlignment(VerticalAlignment.CENTER);
					row = sheet.createRow(i + 3);
					row.setHeightInPoints((float) 13.2);
					// 第四步，创建单元格，并设置值
					vo=list.get(i);
					if(vo!=null&&vo.getName()!=null) {
						row.createCell((short) 0).setCellValue(vo.getName());
						row.getCell(0).setCellStyle(style3);
					}
					if(vo!=null&&vo.getUnlock()!=null) {
						String unLock = "";
						if (vo.getUnlock().equals(0)) {
							unLock="已解锁";
						}else if (vo.getUnlock().equals(1)) {
							unLock="发布及解锁";
						}else {
							unLock=ff.format(vo.getUnlockTime());
						}
						row.createCell((short) 1).setCellValue(unLock);
						row.getCell(1).setCellStyle(style3);
					}				
					if(vo!=null&&vo.getThemeName()!=null) {
						row.createCell((short) 2).setCellValue(vo.getThemeName());
						row.getCell(2).setCellStyle(style3);
					}				
					if(vo!=null&&vo.getRelationCode()!=null) {
						row.createCell((short) 3).setCellValue(vo.getRelationCode());
						row.getCell(3).setCellStyle(style3);
					}
				}
				
			// 第五步，写入实体数据 实际应用中这些数据从数据库得到，
			StringBuffer fileNameSb = new StringBuffer().append(name+"-课程清单").append(sf.format(submitTime)).append(".xlsx");
			String fileName = fileNameSb.toString();
			String path = new StringBuffer().append(requestPath).append(fileNameSb).toString();
			FileOutputStream os=null;
			File file=null;
			try {
				os= new FileOutputStream(path);
				wb.write(os);
				//阿里云返回url
				upLoadUrl = OssUpload.upload(path, fileName);
				file=new File(path);
				success(taskContext,"成功", upLoadUrl);
			} catch (Exception e1) {
				logger.error("写入数据到Excel的过程中或者上传到阿里云中发生错误",e1);	
				fail(taskContext, "写入数据到Excel的过程中或者上传到阿里云中发生错误"+e1.getMessage());
			}
			finally {
				if(os!=null) {
				   os.close();
				}
				if(wb!=null) {
				   wb.close();
				}
				if(file!=null) {
				   file.delete();
				}
			}
		} catch (Exception e) {
			logger.error(name+"专辑课程清单导出过程中发生错误，请查看日志",e);
			fail(taskContext, name+"专辑课程清单导出过程中发生错误，请查看日志"+e.getMessage());	
		}
		return upLoadUrl;
	}

}
